/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in KA_App.java
 * and the file License.html for details.
 */

/*
  Change History:
    01/29/01, JDH: Use new version of Instance.getUseOnts()
    10/26/00, JDH: Used accessor methods for fields in Ontology
 */

package Annotator;

import java.io.*;
import java.awt.*;
import java.util.*;
import UserInt.*;
import SHOE.*;
import java.lang.System;

public class UseOntWindow extends EditWindow {

  public static final int ADD = 1;
  public static final int EDIT = 2;

  Frame parentFrame;
  int winMode;
  Instance edInst;
  Use_Ontology edOnt;
  ObjectList useOntList;
  Label l_id, l_ver, l_pre, l_url;
  TextField idText, verText, prefixText, urlText;
  int edIndex = 0;
  
  OntManager ontman;
  ObjectList instList;

  /* Note: by making elemTree an ObjectList, we have sacrificed the 
     ability to expand and contract categories. */    
  // ElementTree elemTree;            // list of all elements in the ontology
  ObjectList elemTree;            // list of all elements in the ontology
  ObjectList  ontList;
  
  public UseOntWindow(Frame parent, int mode, OntManager om, Instance inst,
		      ObjectList uontL, ObjectList instL) {
    
    super(parent, "Instance");

    parentFrame = parent;
    winMode = mode;
    useOntList = uontL;
    instList=instL;
    edInst = inst;
    ontman=om;

    Panel topPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2));

    // Ontology Panel
    Panel ontPanel = new Panel();
    ontPanel.setLayout(new BorderLayout());
    ontPanel.add("North", new Label("Known Ontologies:"));
    ontList = new ObjectList(10, false);
    ontPanel.add("Center",ontList);
    topPanel.add(ontPanel);

    // Element Panel
    Panel elemPanel = new Panel();
    elemPanel.setLayout(new BorderLayout());
    elemPanel.add("North", new Label("Elements:"));
    // elemTree = new ElementTree(10, false);
    elemTree = new ObjectList(10, false);
    elemPanel.add("Center", elemTree);
    topPanel.add(elemPanel);

    add("North", topPanel);

    Panel textPanel = new Panel();
    textPanel.setLayout(new GridBagLayout());
    l_id = new Label("Id:");
    idText = new TextField(40);
    idText.setBackground(Color.white);
    l_ver = new Label("Version:");
    verText = new TextField(40);
    verText.setBackground(Color.white);
    l_pre = new Label("Prefix:");
    prefixText = new TextField(40);
    prefixText.setBackground(Color.white);
    l_url = new Label("URL:");
    urlText = new TextField(40);
    urlText.setBackground(Color.white);
    Utilities.placeComponent(textPanel, l_id, 0, 0, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(textPanel, idText, 1, 0, 3, 1);
    Utilities.placeComponent(textPanel, l_ver, 0, 1, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(textPanel, verText, 1, 1, 3, 1);
    Utilities.placeComponent(textPanel, l_url, 0, 2, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(textPanel, urlText, 1, 2, 3, 1);
    Utilities.placeComponent(textPanel, l_pre, 0, 3, 1, 1,
			     GridBagConstraints.EAST);
    Utilities.placeComponent(textPanel, prefixText, 1, 3, 3, 1);

    //add("North", textPanel);
    add("Center", textPanel);

    if (winMode == ADD)
      setTitle("Add Use Ontology");
    else {
      setTitle("Edit Use Ontology");
      edIndex = useOntList.getSelectedIndex();
      edOnt = (Use_Ontology)useOntList.getSelectedObject();
      // System.out.println ("Index = " + edIndex + ", Object = " + edOnt);
      idText.setText(edOnt.getId());
      verText.setText(edOnt.getVersion());
      prefixText.setText(edOnt.getPrefix());
      urlText.setText(edOnt.getUrl());

      /* disable fields that user isn't allowed to change in Edit mode */
      ontList.disable();
      elemTree.disable();
    }
    pack();
    resize(minimumSize());

    /* fill in the list of Known Ontologies by querying the Ontology Manager */

    Enumeration enum;
    String label;

    enum = ontman.getKnownOntologies();
    while (enum.hasMoreElements()) {
      label = (String)enum.nextElement();	
      addSortedObjectList(ontList, label, null);
    }
  }

  // The following method is no longer used
  /** Returns a vector of the base objects contained in the ontology 
     argument. Each base object is a SHOE object */
  /*
  public Vector getElement(Ontology ont) {  

    SHOE_Obj obj;
    SHOE_Obj baseObj;        // base object, same as obj except for renames
    Enumeration eltEnum;
    Unique_Elt elmt_id;
    Vector vElem=new Vector();

    eltEnum = ont.getElements();
    while (eltEnum.hasMoreElements()) {
      obj = (SHOE_Obj)eltEnum.nextElement();
      baseObj = obj;
      
      if (obj instanceof Def_Rename) {
	elmt_id = ((Def_Rename)obj).getFromId();
	baseObj = ontman.getElement(elmt_id);
      }

      vElem.addElement(baseObj);    
    }
    return vElem;
  }	
  */
  
  public void  addSortedObjectList(ObjectList list, String label, Object obj) {
    int i;
    int comp;
    boolean bAdded=false;

    for(i=0;i<list.countItems();i++) {
      /* Here's the Java 1.1 version */
      // for(i=0;i<list.getItemCount();i++) {
      /* should run some tests to see if this would be faster if we
	 worked with a vector first. Especially since addItem will
	 continuously resort the List */
	comp=label.compareTo(list.getItem(i));
	if(comp==0) { bAdded=true; break; }
	else if(comp>0) continue;
	else {	
	  list.addItem(label,obj,i);
	  bAdded=true;
	  break;
	}
    }
    if(bAdded==false)
      list.addItem(label,obj);
  }

  /* The following methods, saveChanges(), fieldErrors(), and
     handleEvent() override those in EditWindow */

  public void saveChanges() {

    String id = idText.getText();
    String version = verText.getText();
    String prefix = prefixText.getText();
    String url = urlText.getText();

    if (winMode == ADD) {
      edOnt = new Use_Ontology(id, version, prefix, url);
      ontman.setDefaultUrl(id, version, url);
      ontman.setDefaultPrefix(id, version, prefix);
      useOntList.addItem(edOnt.getOntLabel(), edOnt);
      try {
        edInst.addUseOnt(edOnt);
      } catch (DupeKeyException ex) {
        System.out.println("Prefix is already used. New record not saved");
      }
    }
    else {
      // System.out.println("in saveChanges() for UseOntWindow (edit)");
      edOnt.update(id, version, prefix, url);
      ontman.setDefaultUrl(id, version, url);
      ontman.setDefaultPrefix(id, version, prefix);
      // System.out.println("Replace: " + edOnt.getOntLabel() + ", " + edOnt + ", " + edIndex);
      useOntList.replaceItem(edOnt.getOntLabel(), edOnt, edIndex);
      // System.out.println("after replace item");
    }
  }

  /** Checks that values have been suplied for the Id, Version, URL, and
    Prefix fields. Also determines if the URL specifies a correct location
    for the identified ontology. Note that the ontproxy.dat file may
    specify the location where the system actually looks for the ontology. */
  public String fieldErrors() {

    String idStr = idText.getText();
    String verStr = verText.getText();
    String prefixStr = prefixText.getText();
    String urlStr = urlText.getText();
    String errStr = null;
    Ontology ont;
    ErrorDialog errWin;

    if (idStr.equals(""))
      errStr = "Id field";
    else if (verStr.equals(""))
      errStr = "Version field";
    else if (prefixStr.equals(""))
      errStr = "Prefix field";
    else if (urlStr.equals(""))
      errStr = "URL field";
    if (errStr != null)
      errStr = "You must provide a value for the " + errStr;
    else {
      ont = ontman.getOntology(idStr, verStr, urlStr);
      if (ont == null) {
	errStr = "Cannot find the " + idStr + ", v." + verStr +
	  " ontology at the specified location.";
      }
      else {
	if (ont.hasParseErrors()) {
	  errWin = new ErrorDialog(parentFrame, "Parse Errors", 
                  "The following parse errors were found in " +
		   ont.getOntLabel() + ":", ont.getParseErrors());
	  // Utilities.showDialog((WorkaroundDialog)errWin);
	  ((WorkaroundDialog)errWin).showWorkaround();
	}
      }
    }
    return errStr;
  }
      
  public boolean handleEvent(Event event) {

    if(event.id == Event.LIST_SELECT){
      if(event.target == ontList){
        Ontology ont;
	String label;

	label = ontList.getSelectedItem();
	ont = ontman.getOntology(label);
	if (ont != null) {
	  idText.setText(ont.getId());
	  verText.setText(ont.getVersion()); 
	  urlText.setText(ontman.getDefaultUrl(ont.getId(), ont.getVersion()));
	  prefixText.setText(ontman.getDefaultPrefix(ont.getId(), ont.getVersion()));

	  /* update the Element list to display objects contained in the
	     selected ontology */
	  Enumeration enum;
	  SHOE_Obj obj;
	  String elemName;

	  elemTree.clear();
	  enum = ont.getSortedElements();     // need elements in sorted order
	  while (enum.hasMoreElements()) {
	    obj = (SHOE_Obj)enum.nextElement();
	    elemName = obj.getKey();	
	    elemTree.addItem(elemName, null);
	  }
	}
      }
    }
    return super.handleEvent(event);
  }


}

