/* Work-around Dialog
 * Copyright (C) 1999 - Jeff Heflin - University of Maryland
 *
 * This source code may be redistributed or modified provided that
 * this notice is unchanged and all modifications are clearly indicated.
 * No portion of this code may be included in a commercial product without
 * the written permission of the author.
 *
 * The software and related materials are provided "as-is" without
 * warranty or indemnity of any kind, including any warranties of
 * use, performance or merchantability or fitness for a particular use
 * or purpose or for any purpose whatsoever. User bears all risk
 * relating to use, quality and performance of the software.
 *
 * Author: Jeff Heflin
 * e-mail: heflin@cs.umd.edu
 *
 */

package UserInt;

import java.awt.*;

/** This class is used work around the dialog bugs that occur with
  various implementations of Java. In particular, modal dialogs are
  made to work on Netscape with SunOS or Mac OS. */
public class WorkaroundDialog extends Dialog {

  public final static int DIALOG_PENDING = 0;
  public final static int DIALOG_CLOSED = 1;

  public static int dialogX = 200;
  public static int dialogY = 200;

  private static boolean BlockingWorkaround = false;
  private static boolean ShowWorkaround = false;

  int status = DIALOG_PENDING;

  /* Static Initializer - initializes static variables */
  static {
    String jversion = System.getProperty("java.version");
    String jvendor = System.getProperty("java.vendor");
    String osname = System.getProperty("os.name");
    String osarch = System.getProperty("os.arch");
    if (jversion.startsWith("1.0")) {
      ShowWorkaround = true;
      System.out.println("Using show() workaround");
    }
    /* currently, we only use the blocking workaround for Netscape 3.0
       for Unix or any version of Netscape for Macs. Although the
       bug was fixed in Netscape 4.0 for Unix, as of Netscape 4.5,
       the bug still exists in the Mac. */
    if (jvendor.startsWith("Netscape")) {
      if ((osname.startsWith("Sun") && jversion.startsWith("1.0")) ||
	  (osname.startsWith("Mac"))) {
	BlockingWorkaround = true;
	System.out.println("Using blocking workaround");
      }
    }
  }

  public WorkaroundDialog(Frame parent, String title, boolean modal) {

    super(parent, title, modal);

  }

  /*
  public void show() {

    setStatus(DIALOG_PENDING);

    if (ShowWorkaround == true) {
      ((Window)this).show();         // doesn't work, calls the child method
    }
    else
      super.show();

    if (BlockingWorkaround == true && isModal()) {
      System.out.println("Blocking for dialog");
      while (getStatus() == DIALOG_PENDING);
    }

  }
  */

  /** Sets the dialog's status when the it is closed */
  public void dispose() {

    super.dispose();
    status = DIALOG_CLOSED;
  }

  public void setStatus(int status) {

    this.status = status;
  }

  public int getStatus() {

    return status;
  }

  public static boolean usesBlockingWorkaround() {
    return BlockingWorkaround;
  }

  public static boolean usesShowWorkaround() {
    return ShowWorkaround;
  }

  /** Displays a dialog box that has been modified to work-around problems
     in various implmentations of Java. In particular, Netscape's
     Java version 1.0 don't have dialog windows that inherit the show()
     method. Everywhere where we would normally call d.show(), we instead
     call this function so that it can call the show() method from Window
     if we are using Java 1.0. Also tries to place application popup
     windows closer to the main frame. */
  /* Note: perhaps this method should be moved to WorkaroundDialog */
  public void showWorkaround() {

    // System.out.println("showDialog(WorkaroundDialog)");

    /* There is a tendency for the dialog windows to appear in the upper
       left corner (this can occur with both applets and applications). 
       This is bug number 4102292 in the Java Developer Connection Bug
       list. Here, we check if the window has defaulted to this location,
       and then move it if it is. Note that if we do move it, we must set
       the window's location before calling show() because with modal
       dialogs the code after show() doesn't get
       executed until the window is hidden */
    Point dlgLoc;
    dlgLoc = location();
    // dlgLoc = getLocation();         // for Java 1.1
    if (dlgLoc.x == 0 && dlgLoc.y == 0) {
      move(dialogX, dialogY);
      //setLocation(dialogX, dialogY);        // for Java 1.1
    }

    if (isModal() && usesBlockingWorkaround() == true) {
      setStatus(DIALOG_PENDING);
      System.out.println("Showing a blocking dialog");
    }

    if (ApplType.getIsVersion1_0())
      ((Window)this).show();
    else
      show();

    if (isModal() && usesBlockingWorkaround() == true)
      while (getStatus() == DIALOG_PENDING);

    // System.out.println("showDialog() complete");
  }

}  


